@@ -47,7 +47,10 @@ class Controller(object):
 
         self.artifact_classes = dexy.introspect.artifact_classes(self.log)
         if args.has_key('artifactclass'):
-            self.artifact_class = self.artifact_classes[args['artifactclass']]
+            if self.artifact_classes.has_key(args['artifactclass']):
+                self.artifact_class = self.artifact_classes[args['artifactclass']]
+            else:
+                raise Exception("Artifact class name %s not found in %s" % (args['artifactclass'], ",".join(self.artifact_classes.keys())))
 
     def run(self):
         """
@@ -421,6 +424,34 @@ re.compile: %s""" % (args['except'], e))
             for input_doc in doc.inputs:
                 depend(doc, input_doc)
 
+        if len(self.args['run']) > 0:
+            # Only run the specified document, and its dependencies.
+            new_members = OrderedDict()
+            new_depends = []
+
+            def new_get_pos(member):
+                key = member.key()
+                return new_members.keys().index(key)
+
+            def new_depend(parent, child):
+                new_depends.append((new_get_pos(child), new_get_pos(parent)))
+
+            def parse_new_document(d):
+                new_members[d.key()] = d
+                for input_doc in d.inputs:
+                    if not input_doc.key() in new_members.keys():
+                        new_members[input_doc.key()] = input_doc
+                    new_depend(d, input_doc)
+                    parse_new_document(input_doc)
+
+            run_key = self.args['run']
+            doc = self.members[run_key]
+            parse_new_document(doc)
+
+            print "limiting members list to %s and its dependencies, %s/%s documents will be run" % (doc.key(), len(new_members), len(self.members))
+            self.members = new_members
+            self.depends = new_depends
+
         num_members = len(self.members)
         if num_members > 0:
             dep_ratio = float(total_dependencies)/num_members
