@@ -1,5 +1,6 @@
-from ordereddict import OrderedDict
 from dexy.filters.process_filters import ProcessFilter
+from ordereddict import OrderedDict
+import os
 import pexpect
 import re
 
@@ -9,8 +10,14 @@ class PexpectReplFilter(ProcessFilter):
     """
     ALIASES = ['pexpectreplfilter']
     ALLOW_MATCH_PROMPT_WITHOUT_NEWLINE = False
+    INITIAL_PROMPT = None
     LINE_ENDING = "\r\n"
     PROMPTS = ['>>>', '...'] # Python uses >>> prompt normally and ... when in multi-line structures like loops
+    PROMPT_REGEX = None
+    PS1 = None
+    PS2 = None
+    PS3 = None
+    PS4 = None
     SAVE_VARS_TO_JSON_CMD = None
     TRIM_PROMPT = '>>>'
 
@@ -19,7 +26,10 @@ class PexpectReplFilter(ProcessFilter):
         Search first for the prompt (or prompts) following a line ending.
         Also optionally allow matching the prompt with no preceding line ending.
         """
-        if hasattr(self, 'PROMPT'):
+
+        if self.PROMPT_REGEX:
+            prompts = [self.PROMPT_REGEX]
+        elif hasattr(self, 'PROMPT'):
             prompts = [self.PROMPT]
         else:
             prompts = self.PROMPTS
@@ -55,13 +65,27 @@ class PexpectReplFilter(ProcessFilter):
         if do_record_vars:
             if not self.SAVE_VARS_TO_JSON_CMD:
                 raise Exception("Can't record vars since SAVE_VARS_TO_JSON_CMD not set.")
-            artifact = self.artifact.add_additional_artifact(self.artifact.key + "-vars", 'json')
+            artifact = self.artifact.add_additional_artifact(self.artifact.key + "-vars.json", 'json')
             self.log.debug("Added additional artifact %s (hashstring %s) to store variables" % (artifact.key, artifact.hashstring))
             section_text = self.SAVE_VARS_TO_JSON_CMD % artifact.filename()
             input_dict['dexy--save-vars'] = section_text
+            # TODO this may cause unexpected results if original script only had 1 section
 
         search_terms = self.prompt_search_terms()
+
         env = self.setup_env()
+    	if not env:
+    	    env = os.environ
+
+        if env.has_key('PS1') and self.PS1:
+            env['PS1'] = self.PS1
+        if env.has_key('PS2') and self.PS2:
+            env['PS2'] = self.PS2
+        if env.has_key('PS3') and self.PS3:
+            env['PS3'] = self.PS3
+        if env.has_key('PS4') and self.PS4:
+            env['PS4'] = self.PS4
+
         timeout = self.setup_timeout()
 
         # Spawn the process
@@ -71,7 +95,13 @@ class PexpectReplFilter(ProcessFilter):
                 env=env)
 
         # Capture the initial prompt
-        proc.expect_exact(search_terms, timeout=timeout)
+        if self.INITIAL_PROMPT:
+            proc.expect(self.INITIAL_PROMPT, timeout=timeout)
+        elif self.PROMPT_REGEX:
+            proc.expect(search_terms, timeout=timeout)
+        else:
+            proc.expect_exact(search_terms, timeout=timeout)
+
         start = proc.before + proc.after
         for section_key, section_text in input_dict.items():
             section_transcript = start
@@ -81,7 +111,10 @@ class PexpectReplFilter(ProcessFilter):
             for l in lines:
                 section_transcript += start
                 proc.send(l.rstrip() + "\n")
-                proc.expect_exact(search_terms, timeout=timeout)
+                if self.PROMPT_REGEX:
+                    proc.expect(search_terms, timeout=timeout)
+                else:
+                    proc.expect_exact(search_terms, timeout=timeout)
                 section_transcript += self.strip_newlines(proc.before)
                 start = proc.after
 
@@ -100,13 +133,14 @@ class PexpectReplFilter(ProcessFilter):
 
 class RubyPexpectReplFilter(PexpectReplFilter):
     ALIASES = ['irb', 'rbrepl']
-    ALLOW_MATCH_PROMPT_WITHOUT_NEWLINE = True
-    EXECUTABLE = 'irb --simple-prompt --noreadline'
+    EXECUTABLE = 'irb --simple-prompt'
     IGNORE_ERRORS = True
+    INITIAL_PROMPT = "^>>"
     INPUT_EXTENSIONS = [".txt", ".rb"]
     OUTPUT_EXTENSIONS = [".rbcon"]
     PROMPTS = [">>", "?>"]
-    VERSION = 'irb --version'
+    TRIM_PROMPT = '>>'
+    VERSION_COMMAND = 'irb --version'
 
 class PythonPexpectReplFilter(PexpectReplFilter):
     ALIASES = ['pycon', 'pyrepl']
@@ -114,7 +148,7 @@ class PythonPexpectReplFilter(PexpectReplFilter):
     INPUT_EXTENSIONS = [".txt", ".py"]
     OUTPUT_EXTENSIONS = [".pycon"]
     TAGS = ['python', 'interpreter', 'language']
-    VERSION = 'python --version'
+    VERSION_COMMAND = 'python --version'
 
     SAVE_VARS_TO_JSON_CMD = """
 import json
@@ -128,11 +162,13 @@ dexy__vars_file.close()
 """
 
 class IpythonPexpectReplFilter(PythonPexpectReplFilter):
+    # TODO test for version of ipython that supports --classic
     ALIASES = ['ipython']
     EXECUTABLE = 'ipython --classic'
+    IGNORE_ERRORS = True # TODO try to figure out why we are getting nonzero exit codes
     INPUT_EXTENSIONS = [".txt", ".py"]
     OUTPUT_EXTENSIONS = [".pycon"]
-    VERSION = 'ipython -Version'
+    VERSION_COMMAND = 'ipython -Version'
 
 # untested below here
 
@@ -144,8 +180,8 @@ class RPexpectReplFilter(PexpectReplFilter):
     PROMPTS = [">", "+"]
     TAGS = ['r', 'interpreter', 'language']
     TRIM_PROMPT = ">"
-    VERSION = "R --version"
-    ALLOW_MATCH_PROMPT_WITHOUT_NEWLINE = True
+    INITIAL_PROMPT = "^>"
+    VERSION_COMMAND = "R --version"
     SAVE_VARS_TO_JSON_CMD = """
 if ("rjson" %%in%% installed.packages()) {
     library(rjson)
@@ -162,10 +198,36 @@ class RhinoInteractiveFilter(PexpectReplFilter):
     Runs rhino JavaScript interpeter.
     """
     EXECUTABLE = "rhino"
-    INPUT_EXTENSIONS = [".js"]
-    OUTPUT_EXTENSIONS = [".txt"]
-    ALIASES = ['jsint', 'rhino']
-    PROMPT = "js> "
+    INPUT_EXTENSIONS = [".js", ".txt"]
+    OUTPUT_EXTENSIONS = [".jscon"]
+    ALIASES = ['jsint', 'rhinoint']
+    PROMPTS = ['js>', '  >']
+    TRIM_PROMPT = "js>"
+
+class KshInteractiveFilter(PexpectReplFilter):
+    """
+    Runs ksh. Use to run bash scripts.
+    """
+    ALIASES = ['shint']
+    EXECUTABLE = "ksh -i -r -e"
+    INPUT_EXTENSIONS = [".txt", ".sh"]
+    OUTPUT_EXTENSIONS = ['.sh-session']
+    INITIAL_PROMPT = "^(#|\$)"
+    PROMPTS = ["$", "#"]
+    TRIM_PROMPT = "\$|#"
+    PS1 = "$"
+
+class ZshInteractiveFilter(PexpectReplFilter):
+    """
+    Runs zsh.
+    """
+    ALIASES = ['zshint']
+    EXECUTABLE = "zsh --interactive --restricted --errexit"
+    INPUT_EXTENSIONS = [".txt", ".sh"]
+    OUTPUT_EXTENSIONS = ['.sh-session']
+    PROMPT_REGEX = "[a-z\.]+(%|>)"
+    ALLOW_MATCH_PROMPT_WITHOUT_NEWLINE = True
+    IGNORE_ERRORS = True
 
 class ClojureInteractiveFilter(PexpectReplFilter):
     """
